@@ -4,6 +4,8 @@ use std::collections::HashMap;
 use std::path::Path;
 use tokio::fs;
 
+use crate::transport::{DEFAULT_KEEPALIVE_INTERVAL, DEFAULT_KEEPALIVE_SECS, DEFAULT_NODELAY};
+
 #[derive(Debug, Serialize, Deserialize, Copy, Clone, PartialEq)]
 pub enum TransportType {
     #[serde(rename = "tcp")]
@@ -20,7 +22,7 @@ impl Default for TransportType {
     }
 }
 
-#[derive(Debug, Serialize, Deserialize, Clone, PartialEq)]
+#[derive(Debug, Serialize, Deserialize, Clone, PartialEq, Default)]
 pub struct ClientServiceConfig {
     #[serde(rename = "type", default = "default_service_type")]
     pub service_type: ServiceType,
@@ -28,6 +30,16 @@ pub struct ClientServiceConfig {
     pub name: String,
     pub local_addr: String,
     pub token: Option<String>,
+    pub nodelay: Option<bool>,
+}
+
+impl ClientServiceConfig {
+    pub fn with_name(name: &str) -> ClientServiceConfig {
+        ClientServiceConfig {
+            name: name.to_string(),
+            ..Default::default()
+        }
+    }
 }
 
 #[derive(Debug, Serialize, Deserialize, Clone, Copy, PartialEq)]
@@ -56,8 +68,17 @@ pub struct ServerServiceConfig {
     pub name: String,
     pub bind_addr: String,
     pub token: Option<String>,
+    pub nodelay: Option<bool>,
 }
 
+impl ServerServiceConfig {
+    pub fn with_name(name: &str) -> ServerServiceConfig {
+        ServerServiceConfig {
+            name: name.to_string(),
+            ..Default::default()
+        }
+    }
+}
 #[derive(Clone, Debug, Serialize, Deserialize, PartialEq)]
 pub struct TlsConfig {
     pub hostname: Option<String>,
@@ -79,14 +100,45 @@ pub struct NoiseConfig {
     // TODO: Maybe psk can be added
 }
 
-#[derive(Debug, Serialize, Deserialize, Default, PartialEq, Clone)]
+fn default_nodelay() -> bool {
+    DEFAULT_NODELAY
+}
+
+fn default_keepalive_secs() -> u64 {
+    DEFAULT_KEEPALIVE_SECS
+}
+
+fn default_keepalive_interval() -> u64 {
+    DEFAULT_KEEPALIVE_INTERVAL
+}
+
+#[derive(Debug, Serialize, Deserialize, PartialEq, Clone)]
 pub struct TransportConfig {
     #[serde(rename = "type")]
     pub transport_type: TransportType,
+    #[serde(default = "default_nodelay")]
+    pub nodelay: bool,
+    #[serde(default = "default_keepalive_secs")]
+    pub keepalive_secs: u64,
+    #[serde(default = "default_keepalive_interval")]
+    pub keepalive_interval: u64,
     pub tls: Option<TlsConfig>,
     pub noise: Option<NoiseConfig>,
 }
 
+impl Default for TransportConfig {
+    fn default() -> TransportConfig {
+        TransportConfig {
+            transport_type: Default::default(),
+            nodelay: default_nodelay(),
+            keepalive_secs: default_keepalive_secs(),
+            keepalive_interval: default_keepalive_interval(),
+            tls: None,
+            noise: None,
+        }
+    }
+}
+
 fn default_transport() -> TransportConfig {
     Default::default()
 }
@@ -277,6 +329,7 @@ mod tests {
                 name: "foo1".into(),
                 bind_addr: "127.0.0.1:80".into(),
                 token: None,
+                ..Default::default()
             },
         );
 
@@ -324,6 +377,7 @@ mod tests {
                 name: "foo1".into(),
                 local_addr: "127.0.0.1:80".into(),
                 token: None,
+                ..Default::default()
             },
         );
 
