@@ -5,28 +5,31 @@ use std::{
     time::Duration,
 };
 
-use anyhow::{Context, Result};
+use anyhow::{anyhow, Result};
 use socket2::{SockRef, TcpKeepalive};
-use tokio::net::{TcpStream, ToSocketAddrs, UdpSocket};
-use tracing::error;
+use tokio::net::{lookup_host, TcpStream, ToSocketAddrs, UdpSocket};
+use tracing::trace;
 
 // Tokio hesitates to expose this option...So we have to do it on our own :(
-// The good news is that using socket2 it can be easily done, without losing portablity.
+// The good news is that using socket2 it can be easily done, without losing portability.
 // See https://github.com/tokio-rs/tokio/issues/3082
-pub fn try_set_tcp_keepalive(conn: &TcpStream) -> Result<()> {
+pub fn try_set_tcp_keepalive(
+    conn: &TcpStream,
+    keepalive_duration: Duration,
+    keepalive_interval: Duration,
+) -> Result<()> {
     let s = SockRef::from(conn);
-    let keepalive = TcpKeepalive::new().with_time(Duration::from_secs(60));
-    s.set_tcp_keepalive(&keepalive)
-        .with_context(|| "Failed to set keepalive")
-}
+    let keepalive = TcpKeepalive::new()
+        .with_time(keepalive_duration)
+        .with_interval(keepalive_interval);
 
-pub fn set_tcp_keepalive(conn: &TcpStream) {
-    if let Err(e) = try_set_tcp_keepalive(conn) {
-        error!(
-            "Failed to set TCP keepalive. The connection maybe unstable: {:?}",
-            e
-        );
-    }
+    trace!(
+        "Set TCP keepalive {:?} {:?}",
+        keepalive_duration,
+        keepalive_interval
+    );
+
+    Ok(s.set_tcp_keepalive(&keepalive)?)
 }
 
 #[allow(dead_code)]
@@ -39,12 +42,22 @@ pub fn feature_not_compile(feature: &str) -> ! {
 
 /// Create a UDP socket and connect to `addr`
 pub async fn udp_connect<A: ToSocketAddrs>(addr: A) -> Result<UdpSocket> {
-    // FIXME: This only works for IPv4
-    let s = UdpSocket::bind("0.0.0.0:0").await?;
+    let addr = lookup_host(addr)
+        .await?
+        .next()
+        .ok_or(anyhow!("Failed to lookup the host"))?;
+
+    let bind_addr = match addr {
+        SocketAddr::V4(_) => "0.0.0.0:0",
+        SocketAddr::V6(_) => ":::0",
+    };
+
+    let s = UdpSocket::bind(bind_addr).await?;
     s.connect(addr).await?;
     Ok(s)
 }
 
+// FIXME: These functions are for the load balance for UDP. But not used for now.
 #[allow(dead_code)]
 pub fn hash_socket_addr(a: &SocketAddr) -> u64 {
     let mut hasher = DefaultHasher::new();
@@ -52,7 +65,7 @@ pub fn hash_socket_addr(a: &SocketAddr) -> u64 {
     hasher.finish()
 }
 
-// Wait for the stablization of https://doc.rust-lang.org/std/primitive.i64.html#method.log2
+// Wait for the stabilization of https://doc.rust-lang.org/std/primitive.i64.html#method.log2
 #[allow(dead_code)]
 fn log2_floor(x: usize) -> u8 {
     (x as f64).log2().floor() as u8
@@ -69,8 +82,12 @@ pub fn floor_to_pow_of_2(x: usize) -> usize {
 
 #[cfg(test)]
 mod test {
+    use tokio::net::UdpSocket;
+
     use crate::helper::{floor_to_pow_of_2, log2_floor};
 
+    use super::udp_connect;
+
     #[test]
     fn test_log2_floor() {
         let t = [
@@ -110,4 +127,29 @@ mod test {
             assert_eq!(floor_to_pow_of_2(t.0), t.1);
         }
     }
+
+    #[tokio::test]
+    async fn test_udp_connect() {
+        let hello = "HELLO";
+
+        let t = [("0.0.0.0:2333", "127.0.0.1:2333"), (":::2333", "::1:2333")];
+        for t in t {
+            let listener = UdpSocket::bind(t.0).await.unwrap();
+
+            let handle = tokio::spawn(async move {
+                let s = udp_connect(t.1).await.unwrap();
+                s.send(hello.as_bytes()).await.unwrap();
+                let mut buf = [0u8; 16];
+                let n = s.recv(&mut buf).await.unwrap();
+                assert_eq!(&buf[..n], hello.as_bytes());
+            });
+
+            let mut buf = [0u8; 16];
+            let (n, addr) = listener.recv_from(&mut buf).await.unwrap();
+            assert_eq!(&buf[..n], hello.as_bytes());
+            listener.send_to(&buf[..n], addr).await.unwrap();
+
+            handle.await.unwrap();
+        }
+    }
 }
